﻿using SCClient;
using System;
using System.Linq;

namespace QPR.QPRSuite.AcceleratorTools {
    public class MetricsClient : IMetricsClient {

        private SCApplication client = null;
        private SCModel model = null;
        private IContextManager context = null;
        private int openModelId = 0;
        public static int DEFAULT_PORT = 20761;

        public MetricsClient(IContextManager context) {
            this.context = context;
        }

        private void checkClientOpened() {
            if (this.client == null) {
                this.client = new SCApplication();
                int returnCode;
                string port = this.context.getOptionalApplicationSetting("metricsserverport");
                if (string.IsNullOrEmpty(port)) port = MetricsClient.DEFAULT_PORT + "";
                returnCode = this.client.ConnectToServer(this.context.getMandatoryNotEmptyApplicationSetting("servername"), Int32.Parse(port));
                checkReturnCode(returnCode, "connecting to server");
                string uiMode = this.context.getOptionalApplicationSetting("showclient");
                if (string.IsNullOrEmpty(uiMode)) uiMode = "0";
                this.client.SetUIMode(Int32.Parse(uiMode));
                returnCode = this.client.Authenticate(this.context.getMandatoryNotEmptyApplicationSetting("username"), this.context.getMandatoryApplicationSetting("password"), this.context.getOptionalApplicationSetting("domain"));
                checkReturnCode(returnCode, "authenticating to server");
            }
        }

        public void openModel(int modelId) {
            this.checkClientOpened();
            int returnCode;
            object outModel;
            returnCode = this.client.OpenModel(modelId, out outModel);
            checkReturnCode(returnCode, "opening model by id");
            this.model = (SCModel)outModel;
        }

        private void closeModel() {
            if (this.model == null) return;
            int returnCode;
            returnCode = this.model.CloseModel();
            checkReturnCode(returnCode, "closing model");
            this.model = null;
        }

        private void checkModelOpened(int modelId) {
            this.checkClientOpened();
            if (modelId == 0 && this.openModelId != 0) { //use the currently open model
                return;
            } else if (this.openModelId == 0) { //no model open
                this.openModel(modelId);
                this.openModelId = modelId;
            } else if (this.openModelId != modelId) { //wrong model open
                this.closeModel();
                this.openModel(modelId);
                this.openModelId = modelId;
            }
        }

        public void refreshDrillDown(int modelId, int targetObjectId) {
            this.checkModelOpened(modelId);
            if (this.model == null) throw new Exception("Model is not opened");
            int returnCode;
            returnCode = this.model.RefreshDrillDown(targetObjectId);
            checkReturnCode(returnCode, "RefreshDrillDown (targetObjectId=\"" + targetObjectId + "\")");
        }

        public void applyInheritanceProperties(int modelId, int scorecardId, string applyParameter, int applyWholeChain) {
            this.checkModelOpened(modelId);
            if (this.model == null) throw new Exception("Model is not opened");
            int returnCode;
            returnCode = this.model.ApplyInheritanceProperties(scorecardId, applyParameter, applyWholeChain);
            checkReturnCode(returnCode, "ApplyInheritanceProperties (scorecardId=\"" + scorecardId + "\", applyParameter=\"" + applyParameter + "\", applyWholeChain=\"" + applyWholeChain + "\")");
        }

        public void recalculate(int modelId, int periodId, object date) {
            this.checkModelOpened(modelId);
            if (this.model == null) throw new Exception("Model is not opened");
            int returnCode;
            returnCode = this.model.Recalculate(periodId, date);
            checkReturnCode(returnCode, "ApplyInheritanceProperties (periodId=\"" + periodId + "\", date=\"" + date + "\")");
        }

        public int getModelIdByName(string modelName) {
            object models;
            int returnCode;
            returnCode = this.client.GetModels(out models);
            checkReturnCode(returnCode, "getting models");
            return 0;
        }

        private void checkReturnCode(int returnCode, string task) {
            this.context.getLogger().writeLog((task.ElementAt(0) + "").ToUpper() + task.Substring(1) + " (" + returnCode + ")", LogLevel.verbose);
            if (returnCode != 0) {
                throw new Exception("QPR Metrics Client returned an error code " + returnCode + ". Error occurred when " + task + ". (http://kb.qpr.com/qpr2015-1/index.html?returnvalues.htm)");
            }
        }

        public void Dispose() {
            if (this.client != null) {
                this.client.Quit();
            }
            this.model = null;
            this.client = null;
        }

    }
}
