﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using QPR.QPRSuite.AcceleratorTools.QPRWebService;
using System.IO;
using QPR.QPRSuite.NCalc;

namespace QPR.QPRSuite.AcceleratorTools {

    public class WebServiceClient : IWebServiceClient {
        private ServiceClient webServiceClient;
        private string webServiceSessionId;
        private IContextManager context;
        private WCFSecurityMode WSFSecurityMode;
        private QPRAuthenticationMode QPRAuthenticationMode;
        private string webServiceAddress;
        private string username;
        private string password;
        private string webServiceEndpointName;
        public static string XML_DATEFORMAT = "yyyy-MM-ddTHH:mm:ss.fffzzz";

        public WebServiceClient(IContextManager context, WCFSecurityMode WSFSecurityMode, QPRAuthenticationMode QPRAuthenticationMode, string webServiceSessionId, string webServiceAddress, string username, string password, string webServiceEndpointName) {
            this.context = context;
            this.WSFSecurityMode = WSFSecurityMode;
            this.QPRAuthenticationMode = QPRAuthenticationMode;
            this.webServiceSessionId = webServiceSessionId;
            this.webServiceAddress = webServiceAddress;
            this.username = username;
            this.password = password;
            this.webServiceEndpointName = webServiceEndpointName;
        }

        private void authenticate() {
            if (string.IsNullOrEmpty(this.webServiceEndpointName)) throw new Exception("Configured QPR Web Service endpoint name is empty");
            if (string.IsNullOrEmpty(this.webServiceAddress)) throw new Exception("Configured QPR Web Service address is empty");
            this.webServiceClient = new ServiceClient(this.webServiceEndpointName, this.webServiceAddress);
            if (this.WSFSecurityMode == WCFSecurityMode.none) {
                ((WSHttpBinding)this.webServiceClient.Endpoint.Binding).Security.Mode = SecurityMode.None;
            } else if (this.WSFSecurityMode == WCFSecurityMode.message) {
                ((WSHttpBinding)this.webServiceClient.Endpoint.Binding).Security.Mode = SecurityMode.Message;
            } else if (this.WSFSecurityMode == WCFSecurityMode.transport) {
                ((WSHttpBinding)this.webServiceClient.Endpoint.Binding).Security.Mode = SecurityMode.Transport;
            } else if (this.WSFSecurityMode == WCFSecurityMode.transportwithmessagecredential) {
                ((WSHttpBinding)this.webServiceClient.Endpoint.Binding).Security.Mode = SecurityMode.TransportWithMessageCredential;
            }

            if (this.QPRAuthenticationMode == QPRAuthenticationMode.password && string.IsNullOrEmpty(this.webServiceSessionId)) {
                try {
                    this.webServiceSessionId = this.webServiceClient.Authenticate(this.username, this.password);
                } catch (Exception e) {
                    throw new Exception("Error in Web Service's Authenticate method\r\nUsername: " + this.username + "\r\nAddress: " + this.webServiceAddress, e);
                }
            } else if (this.QPRAuthenticationMode == QPRAuthenticationMode.passedsession) {
                if (string.IsNullOrEmpty(this.webServiceSessionId)) throw new Exception("QPR Web service's session id is not passed as a parameter.");
            }
        }

        public void Dispose() {
            if (this.webServiceClient != null) {
                try {
                    this.webServiceClient.Close();
                } catch (Exception e) { }
            }
        }

        public string getAttributeAsString(string objectId, string attribute, string options) {
            attribute = ExpressionEngine.replaceQuotationMarks(attribute);
            options = ExpressionEngine.replaceQuotationMarks(options);
            if (this.webServiceClient == null) this.authenticate();
            string returnValue;
            try {
                returnValue = this.webServiceClient.GetAttributeAsString(this.webServiceSessionId, objectId, attribute, options);
            } catch (Exception e) {
                throw new Exception("Error in Web Service's GetAttributeAsString with parameters: objectid=\"" + objectId + "\", attribute=\"" + attribute + "\", options=\"" + options + "\"", e);
            }
            if (returnValue == null) returnValue = "";
            this.context.getLogger().writeLog("Web Service GetAttributeAsString: sessionid=\"" + this.webServiceSessionId + "\", objectid=\"" + objectId + "\", attribute=\"" + attribute + "\", options=\"" + options + "\", returnvalue=\"" + returnValue + "\"", LogLevel.verbose);
            return returnValue;
        }

        public ResultSet queryObjects(string query, string criteria, string sortBy, string attributes, string options) {
            query = ExpressionEngine.replaceQuotationMarks(query);
            criteria = ExpressionEngine.replaceQuotationMarks(criteria);
            sortBy = ExpressionEngine.replaceQuotationMarks(sortBy);
            attributes = ExpressionEngine.replaceQuotationMarks(attributes);
            options = ExpressionEngine.replaceQuotationMarks(options);
            if (this.webServiceClient == null) this.authenticate();
            ResultSet returnValue;
            try {
                returnValue = this.webServiceClient.QueryObjects(this.webServiceSessionId, query, criteria, sortBy, attributes, options);
            } catch (Exception e) {
                throw new Exception("Error in Web Service's QueryObjects with parameters: query=\"" + query + "\", criteria=\"" + criteria + "\", sortBy=\"" + sortBy + "\", attributes=\"" + attributes + "\", options=\"" + options + "\"\r\nAddress: " + this.context.getMandatoryApplicationSetting("qprwebserviceaddress"), e);
            }
            this.context.getLogger().writeLog("Web Service QueryObjects: sessionid=\"" + this.webServiceSessionId + "\", query=\"" + query + "\", criteria=\"" + criteria + "\", sortBy=\"" + sortBy + "\", attributes=\"" + attributes + "\", options=\"" + options + "\", returnvalue=\"" + showQueryObjectsResultAsString(returnValue) + "\"", LogLevel.verbose);
            return returnValue;
        }

        public string getPortalUrl(string objectId, string view, string options) {
            view = ExpressionEngine.replaceQuotationMarks(view);
            options = ExpressionEngine.replaceQuotationMarks(options);
            if (this.webServiceClient == null) this.authenticate();
            string url = null;
            try {
                url = this.webServiceClient.GetPortalUrl(this.webServiceSessionId, objectId, view, options);
            } catch (Exception e) {
                throw new Exception("Error in Web Service's GetPortalUrl with parameters: objectid=\"" + objectId + "\", view=\"" + view + "\", options=\"" + options + "\"", e);
            }
            this.context.getLogger().writeLog("Web Service GetPortalUrl: sessionid=\"" + this.webServiceSessionId + "\", objectid=\"" + objectId + "\", view=\"" + view + "\", options=\"" + options + "\" returnvalue=\"" + url + "\"", LogLevel.verbose);
            return url;
        }

        public string createObject(string namespaceId, string typename, string name, string parentIds, string[][] attributes, string options) {
            name = ExpressionEngine.replaceQuotationMarks(name);
            options = ExpressionEngine.replaceQuotationMarks(options);
            if (this.webServiceClient == null) this.authenticate();
            string createdObjectId = null;
            try {
                createdObjectId = this.webServiceClient.CreateObject(this.webServiceSessionId, namespaceId, typename, name, parentIds, attributes, options);
            } catch (Exception e) {
                throw new Exception("Error in Web Service's CreateObject with parameters: namespaceid=\"" + namespaceId + "\", typename=\"" + typename + "\", name=\"" + name + "\", parentIds=\"" + parentIds + "\", options=\"" + options + "\"", e);
            }
            this.context.getLogger().writeLog("Web Service CreateObject: sessionid=\"" + this.webServiceSessionId + "\", namespaceid=\"" + namespaceId + "\", typename=\"" + typename + "\", name=\"" + name + "\", parentIds=\"" + parentIds + "\", options=\"" + options + "\"", LogLevel.verbose);
            return createdObjectId;
        }

        public void setAttribute(string objectId, string attribute, string value, string options) {
            options = ExpressionEngine.replaceQuotationMarks(options);
            if (this.webServiceClient == null) this.authenticate();
            try {
                this.webServiceClient.SetAttribute(this.webServiceSessionId, objectId, attribute, value, options);
            } catch (Exception e) {
                throw new Exception("Error in Web Service's SetAttribute with parameters: objectid=\"" + objectId + "\", attribute=\"" + attribute + "\", value=\"" + value + "\", options=\"" + options + "\"", e);
            }
            this.context.getLogger().writeLog("Web Service SetAttribute: sessionid=\"" + this.webServiceSessionId + "\", objectid=\"" + objectId + "\", attribute=\"" + attribute + "\", value=\"" + value + "\", options=\"" + options + "\"", LogLevel.verbose);
        }

        public void setAttributes(string objectIds, string[][] attributes, string options) {
            options = ExpressionEngine.replaceQuotationMarks(options);
            if (this.webServiceClient == null) this.authenticate();
            try {
                this.webServiceClient.SetAttributes(this.webServiceSessionId, objectIds, attributes, options);
            } catch (Exception e) {
                throw new Exception("Error in Web Service's SetAttribute with parameters: objectid=\"" + objectIds + "\", options=\"" + options + "\"", e);
            }
            this.context.getLogger().writeLog("Web Service SetAttribute: sessionid=\"" + this.webServiceSessionId + "\", objectid=\"" + objectIds + "\", options=\"" + options + "\"", LogLevel.verbose);
        }

        public BinaryData getBinaryData(string objectId, string attribute, string options) {
            options = ExpressionEngine.replaceQuotationMarks(options);
            if (this.webServiceClient == null) this.authenticate();
            BinaryData resultValue;
            try {
                resultValue = this.webServiceClient.GetBinaryData(this.webServiceSessionId, objectId, attribute, options);
            } catch (Exception e) {
                throw new Exception("Error in Web Service's GetBinaryData with parameters: objectid=\"" + objectId + "\", attribute=\"" + attribute + "\", options=\"" + options + "\"", e);
            }
            this.context.getLogger().writeLog("Web Service GetBinaryData: sessionid=\"" + this.webServiceSessionId + "\", objectid=\"" + objectId + "\", attribute=\"" + attribute + ", options=\"" + options + "\"", LogLevel.verbose);
            return resultValue;
        }

        public BinaryData getGraph(string objectId, string options) {
            options = ExpressionEngine.replaceQuotationMarks(options);
            if (this.webServiceClient == null) this.authenticate();
            BinaryData resultValue;
            try {
                resultValue = this.webServiceClient.GetGraph(this.webServiceSessionId, objectId, options);
            } catch (Exception e) {
                throw new Exception("Error in Web Service's GetGraph with parameters: objectid=\"" + objectId + "\", options=\"" + options + "\"", e);
            }
            this.context.getLogger().writeLog("Web Service GetGraph: sessionid=\"" + this.webServiceSessionId + "\", objectid=\"" + objectId + ", options=\"" + options + "\"", LogLevel.verbose);
            return resultValue;
        }

        public Stream getGraphAsStream(string objectId, string options) {
            options = ExpressionEngine.replaceQuotationMarks(options);
            if (this.webServiceClient == null) this.authenticate();
            Stream resultValue;
            try {
                resultValue = this.webServiceClient.GetGraphAsStream(this.webServiceSessionId, objectId, options);
            } catch (Exception e) {
                throw new Exception("Error in Web Service's GetGraph with parameters: objectid=\"" + objectId + "\", options=\"" + options + "\"", e);
            }
            this.context.getLogger().writeLog("Web Service GetGraph: sessionid=\"" + this.webServiceSessionId + "\", objectid=\"" + objectId + ", options=\"" + options + "\"", LogLevel.verbose);
            return resultValue;
        }

        public void deleteObject(string objectId, string options) {
            options = ExpressionEngine.replaceQuotationMarks(options);
            if (this.webServiceClient == null) this.authenticate();
            try {
                this.webServiceClient.DeleteObject(this.webServiceSessionId, objectId, options);
            } catch (Exception e) {
                throw new Exception("Error in Web Service's DeleteObject with parameters: objectid=\"" + objectId + "\", options=\"" + options + "\"", e);
            }
            this.context.getLogger().writeLog("Web Service DeleteObject: sessionid=\"" + this.webServiceSessionId + "\", objectid=\"" + objectId + "\", options=\"" + options + "\"", LogLevel.verbose);
        }

        private static string showQueryObjectsResultAsString(ResultSet resultSet) {
            List<string> stringList = new List<string>();
            if (resultSet.ChildElements == null || resultSet.ChildElements.Length == 0) return "";
            foreach (ResultObject ro in resultSet.ChildElements) {
                List<string> attributeList = new List<string>();
                if (ro.ChildElements != null && ro.ChildElements.Length > 0) {
                    foreach (ValueElement ve in ro.ChildElements) {
                        string value = ve.Value;
                        if (value == null) value = "";
                        attributeList.Add(ve.Name + "=\"" + value.Replace("\r\n", "<linebreak>") + "\"");
                    }
                }
                stringList.Add(ro.Id + "(" + String.Join(", ", attributeList) + ")");
            }
            return String.Join("|", stringList);
        }

        public string getWebServiceSessionId() {
            if (this.webServiceClient == null) this.authenticate();
            return this.webServiceSessionId;
        }

    }
}
